"""
@author Vitaliy Bohera
@email vitaliyb@students.zcu.cz
@version 1
@task Zadani cislo 15 - Implementace znalostniho systemu pro pripravu vecere (pokrm Kung-pao)
@info Inferencni modul znalostniho systemu pro pripravu Kung-pao
@date 15.12.2023
"""
import komunikace
import vysvetlovaci_modul
import znalosti


'''
Funkce spusti znalostni system
'''
def start_system():
    komunikace.uvod_do_systemu()                        # vypsani hlavicky
    ptej_se_na_suroviny()                               # ptej se uzivatele na suroviny
    vysvetlovaci_modul.vysvetli_obsah_uzivatele()       # vypis zjistene informace
    zobrazit_recept = komunikace.zeptej_se_na_recept()
    if zobrazit_recept:
        komunikace.zobraz_recept()                      # na zadost uzivatele vypis navod na pripravu Kung-pao


'''
Funkce se pta uzivatele na jednotlive suroviny
pokud uzivatel ma surovinu -> pridava ji k seznamu surovin uzivatele
pokud uzivatel nema surovinu -> pta se, jestli nema nahradu za tuto surovinu
pokud uzivatel zada spatny vstup -> vypise chybovou hlasku a pta se znovu
'''
def ptej_se_na_suroviny():
    suroviny = znalosti.get_suroviny()
    for surovina in suroviny:
        zprava = komunikace.zeptej_se_na_surovinu(surovina)
        ma = kontrola_vstupu(zprava)
        while ma == -1:
            zprava = komunikace.zeptej_se_na_surovinu(surovina)
            ma = kontrola_vstupu(zprava)
        if ma == 1:
            vysvetlovaci_modul.vysvetli_pridani_suroviny(surovina)
            print()
            continue
        else:
            ptej_se_na_nahradu(surovina)


'''
Funkce se pta uzivatele zda ma nahradu za surovinu
pokud pro surovinu nejsou nahrady nebo uzivatel nema jeji nahradu -> musi koupit danou surovinu v obchode
pokud uzivatel ma nahradu za tuto surovinu -> pridava ji k seznamu nahrad uzivatele
pokud uzivatel zada spatny vstup -> vypise chybovou hlasku a pta se znovu
@param surovina - surovina, pro kterou najdeme nahradu a ptame se, zda ji ma uzivatel
'''
def ptej_se_na_nahradu(surovina):
    nahrada = vysvetlovaci_modul.vysvetli_nahradu(surovina)
    if nahrada is None:
        koupit_v_obchode(surovina)
        return
    zprava = komunikace.zeptej_se_na_nahradu(nahrada)
    ma = kontrola_vstupu(zprava)
    while ma == -1:
        zprava = komunikace.zeptej_se_na_nahradu(nahrada)
        ma = kontrola_vstupu(zprava)
    if ma == 1:
        vysvetlovaci_modul.vysvetli_pridani_nahrady(nahrada, surovina)
        print()
        return
    else:
        koupit_v_obchode(surovina)


'''
Funkce informuje uzivatele o tom, ze potrebuje koupit danou surovinu v obchode
a pridava ji k nakupnimu seznamu
@param surovina - surovina, kterou uzivatel potrebuje dokoupit v obchode
'''
def koupit_v_obchode(surovina):
    obchod = vysvetlovaci_modul.vysvetli_obchod(surovina)
    znalosti.pridej_surovinu_z_obchodu(surovina, obchod)


'''
Funkce kontroluje vsup od uzivatele
pokud vstup neodpovida formatu -> vypise chybovou hlasku
@param vstup - vstup od uzivatele
@return 1 - pokud uzivatel zada ANO, A, ano nebo Ano
        0 - pokud uzivatel zada NE, N, ne nebo Ne
        -1 - jinak, indikuje spatny vstup
'''
def kontrola_vstupu(vstup):
    if vstup == "ANO" or vstup == "ano" or vstup == "Ano" or vstup == "A":
        return 1
    elif vstup == "NE" or vstup == "ne" or vstup == "Ne" or vstup == "N":
        return 0
    else:
        komunikace.spatny_vstup()
        return -1
