"""
Baze dat systemu. Obsahuje definici dat.
Semestralni prace Lloydova 15
autor: Miroslav Martinek
e-mail: mira2002@students.zcu.cz
datum: 20.2.2023
verze: 2
"""
import random


# trida reprezentujici stav hry
# ma metody pracujici se svymi daty
class GameState:
    # inicializace pri ktere predame rodice(koren stromu dostava None)
    def __init__(self, parent):
        self.parent = parent
        self.field = None
        self.x = 0
        self.y = 0
        self.rating = 0

    # primitivni porovnani, ktere pouziva funkce ze standardni knihovny bisect
    def __lt__(self, other):
        return self.rating > other.rating

    # objekt ze sebe udela jednim tahem zmenenou verzi rodice
    # vstupni souradnice urcuji kde skonci prazdne policko
    def modification(self, new_y, new_x):
        self.field = [row[:] for row in self.parent.field]
        self.y = self.parent.y
        self.x = self.parent.x
        temp = self.field[self.y][self.x]
        self.field[self.y][self.x] = self.field[new_y][new_x]
        self.field[new_y][new_x] = temp
        self.y = new_y
        self.x = new_x

    # objekt ze sebe udela nahodny stav k pouziti jako pocatecni stav
    def root(self, n):
        nums = [*range(1, n*n + 1)]
        random.shuffle(nums)  # nahodne rozmistene cisla od 1 do n*n+1
        self.field = []
        for x in range(n):
            self.field.append(nums[x*n:x*n+n])  # pridani radky cisel vybrane z nums do 2d pole
        for row in range(n):
            for column in range(n):
                if(self.field[row][column] == n*n):  # nalezeni souradnic volneho mista
                    self.x = column
                    self.y = row

    # ohodnocovaci funkce, kde ohodnoceni je soucet Manhattan vzdalenosti cisel od svich spravnych souradnic
    def rate(self, n):
        for row in range(n):
            for column in range(n):
                correct_row = int((self.field[row][column] - 1) / n)
                correct_column = (self.field[row][column] - 1) % n
                self.rating += abs(row - correct_row) + abs(column - correct_column)

    # vytvori sve potomky "posunem" cisel vedle volneho mista
    # vraci list 2-4 potomku (<4 kdyz volne misto na kraji herniho pole)
    def make_children(self, n):
        children = []
        if(self.x > 0):
            child = GameState(self)
            child.modification(self.y, self.x - 1)
            child.rate(n)
            children.append(child)
        if (self.x < n - 1):
            child = GameState(self)
            child.modification(self.y, self.x + 1)
            child.rate(n)
            children.append(child)
        if (self.y > 0):
            child = GameState(self)
            child.modification(self.y - 1, self.x)
            child.rate(n)
            children.append(child)
        if (self.y < n - 1):
            child = GameState(self)
            child.modification(self.y + 1, self.x)
            child.rate(n)
            children.append(child)
        return children
