"""
Baze znalosti systemu. Obsahuje funkce pro praci s daty.
Semestralni prace Lloydova 15
autor: Miroslav Martinek
e-mail: mira2002@students.zcu.cz
datum: 20.2.2023
verze: 2
"""
from baze_dat import *
import bisect


# porovnani zda dva stavy jsou stejne obsahem
# vraci jsou stejne(True)/nejsou stejne(False)
def same_state(state1: GameState, state2: GameState, n):
    # stavy, ktere nemaji stejne ohodnoceni nebudou stejne
    if(state1.rating != state2.rating):
        return False
    # porovnani souradnic volneho mista
    if (state1.x != state2.x):
        return False
    if (state1.y != state2.y):
        return False
    # porovnani hernich poli
    for row in range(n):
        for column in range(n):
            if(state1.field[row][column] != state2.field[row][column]):
                return False
    return True


# zjisteni resitelnosti (pocatecniho) herniho stavu
# vraci resitelne(True)/neresitelne(False)
def solvability(state: GameState, n):
    inversions = 0
    square = n*n
    # spocitame inverze(situace kdy se vetsi cislo nachazi pred mensim) pro kazde cislo v poli
    for start_index in range(square):
        if(state.field[int(start_index / n)][start_index % n] == square):  # volne policko ignorujeme
            continue
        for compare_index in range(start_index + 1, square):
            if(state.field[int(start_index / n)][start_index % n] >
                    state.field[int(compare_index / n)][compare_index % n]):
                inversions += 1
    # rozhodnuti o resitelnosti podle lichosti/sudosti velikosti pole, inverzi a lokaci volneho mista
    if(n % 2 == 1 and inversions % 2 == 0):
        return True
    elif(n % 2 == 0 and (inversions + state.y) % 2 == 1):
        return True
    return False


# vytvori pocatecni stav podle obsahu root_info a velikosti hraciho pole N
# vraci vytovoreny pocatecni stav(GameState)
def prepare_root(root_info, n):
    root = GameState(None)
    if(root_info is None):  # nahodne generujeme koren
        root.root(n)
    else:  # koren zadan manualne uzivatelem
        root.field = root_info[0]
        root.x = root_info[1]
        root.y = root_info[2]
    root.rate(n)
    return root


# hledani reseni pro pocatecni stav root
# vraci pocatecni stav neresitelny(None)/nalezeny konecny stav(GameState objekt)
def lloyds_n(root, n):
    explored = {}  # "closed list" implementovany jako slovnik pro rychlejsi hledani
    candidates = []  # open list serazeny podle ohodnoceni stavu klesajicim zpusobem(nejlepsi stav ma index -1)

    # rozhodnuti o resitelnosti
    if(not solvability(root, n)):
        return None

    candidates.append(root)

    X_FACTOR = 4
    Y_FACTOR = 5
    # closed slovnik obsahuje listy hernich stavu, zde pridame prazdne listy
    # vysvetleni "n*X_FACTOR + n*Y_FACTOR + (n*n*2*(n-1))":
    # herni stavy chceme do slovniku trochu rozprostrid na zaklade x/y souradnic volneho policka a ohodnoceni
    # souradnice jsou mnohem mensi cisla nez ohodnoceni a vetsina closed stavu ma ohodnoceni v malem rosahu
    # pro lepsi rozlozeni souradnice nasobime, aby meli vetsi vliv
    # n*X_FACTOR - volne misto na pravem kraji
    # n*Y_FACTOR - volne misto na spodnim kraji
    # n*n*2*(n-1) - teoreticke ohodnoceni stavu, kde vsechny cisla(n*n) maji max Manhattan vzdalenost(2*(n-1))
    # do slovniku tak muzeme pristupovat beze strachu ze program spadne
    for x in range(n*X_FACTOR + n*Y_FACTOR + (n*n*2*(n-1))):
        explored[x] = []
    # hlavni smicka hledani koncoveho stavu
    while(len(candidates)):
        if(candidates[-1].rating == 0):  # koncovi stav nalezen
            return candidates[-1]
        children = candidates[-1].make_children(n)  # pro nejlepsi stav vytvorime potomky

        explored[candidates[-1].x * X_FACTOR + candidates[-1].y * Y_FACTOR + candidates[-1].rating]\
            .append(candidates.pop())  # expandovany stav presuneme do closed slovniku
        # odfiltrujeme z potomku stavy, pro ktere je ekvivalentni stav v closed slovniku
        filtered = children.copy()
        for child in children:
            for suspect in explored[child.x * X_FACTOR + child.y * Y_FACTOR + child.rating]:
                if(same_state(child, suspect, n)):
                    filtered.remove(child)
                    break
        children = filtered

        # do open listu vlozime potomky potomky, pri vkladani insort udrzuje serazeni
        for child in children:
            bisect.insort(candidates, child)
