"""
Description: Tento modul řídí běh znalostního systému.
Author: Václav Ovsík
E-mail: ovsikv@students.zcu.cz
Version: 1.0
Date: 18.12.2023
"""
import random

import komunikacni_modul
import baze_dat
import baze_znalosti
import vysvetlovaci_modul


def start():
    """
    Spouští ostatní metody. Jedná se o první spuštěnou metodou modulem main.
    """
    komunikacni_modul.start_info()
    get_data()
    game_loop()
    if komunikacni_modul.end_info() == "a":
        start()
    else:
        return


def game_loop():
    """
    Hlavní herní smyčka která kontroluje tahy hráčů.
    """
    winner = False  # True - pokud je uživatel výhercem
    while not is_finished(baze_dat.board):
        komunikacni_modul.print_board(baze_dat.board)
        pile_indices, count_removed = get_move(baze_dat.board)
        vysvetlovaci_modul.print_help(count_removed, pile_indices)
        match_count = komunikacni_modul.user_input_mc()

        # dokud není zadán validní vstup, ptáme se na vstup znovu
        while True:
            pile_numbers = komunikacni_modul.user_input_pn()
            if len(pile_numbers) == 1:
                # odebírání z jedné hromádky
                if take_from_one(match_count, pile_numbers[0], baze_dat.board):
                    break
                komunikacni_modul.input_error()
            else:
                # odebírání ze dvou hromádek
                if take_from_two(match_count, pile_numbers[0], pile_numbers[1], baze_dat.board):
                    break
                komunikacni_modul.input_error()

        # kontrola zda hru vyhrál uživatel
        if is_finished(baze_dat.board):
            winner = True
            break
        komunikacni_modul.print_board(baze_dat.board)
        pile_indices, count_removed = get_move(baze_dat.board)   # zjistí optimální tah
        if len(pile_indices) == 1:
            take_from_one(count_removed, pile_indices[0], baze_dat.board)
        else:
            take_from_two(count_removed, pile_indices[0], pile_indices[1], baze_dat.board)
        komunikacni_modul.print_opponent_move(pile_indices, count_removed)
    komunikacni_modul.print_winner(winner)


def get_move(board):
    """
    Zjistí optimální tah pro aktuální stav.
    :param board: aktuální stav
    :return: seznam indexů hromádek ze kterých se bude odebírat, počet odebíraných zápalek
    """
    nim_sum = calculate_nim_sum(board)
    # print(nim_sum)
    if nim_sum != 0:
        # nim_sum není nulová, snažíme se ji vynulovat
        for i in range(len(board)):
            if (board[i] ^ nim_sum) < board[i]:
                count_removed = board[i] - (board[i] ^ nim_sum)
                return [i], count_removed
    else:
        # nim_sum je nulová, nelze ji udržet nulovou standartním tahem, takže uděláme náhodný validní tah
        indices = []

        # vybere z nenulových hromádek jednu, ze které odebere náhodný validní počet zápalek
        if len(indices) == 0:
            non_zero_indices = []
            for i in range(len(board)):
                if board[i] > 0:
                    non_zero_indices.append(i)

            pile_index = random.randint(0, len(non_zero_indices)-1)
            count_removed = random.randint(1, board[non_zero_indices[pile_index]])
            return [non_zero_indices[pile_index]], count_removed


def calculate_nim_sum(board):
    """
    Vypočte Nim-Sum aktuálního stavu hry.
    :param board: stav hry (seznam počtů zápalek v hromádkách)
    :return: Nim-Sum aktuálního stavu hry
    """
    nim_sum = 0
    for count in board:
        nim_sum ^= count
    return nim_sum


def get_data():
    """
    Získá úvodní data od uživatele a zapíše je do báze dat.
    """
    pile_count = komunikacni_modul.user_input_pc()
    baze_dat.n = pile_count
    baze_dat.board = innit_board()


def innit_board():
    """
    Inicializuje herní plochu.
    :return: úvodní stav hry (jako seznam)
    """
    board = []
    for num in range(baze_dat.n):
        board.insert(num, 1 + (2 * num))
    return board


def is_finished(board):
    """
    Zjišťuje zda je hra v koncovém stavu
    :param board: stav hry (seznam počtů zápalek v hromádkách)
    :return: True pokud je v koncovém stavu
             False pokud není v koncovém stavu
    """
    if sum(board) == 0:
        return True
    else:
        return False


def take_from_one(count, pile_index, board):
    """
    Pokusí se odebrat daný počet zápalek z dané hromádky.
    :param count: počet odebíraných zápalek
    :param pile_index: index hromádky
    :param board: aktuální stav hry
    :return: True pokud se podařilo
             False pokud se nepodařilo
    """
    if not baze_znalosti.can_take_from_one(count, pile_index, board):
        return False
    board[pile_index] -= count
    return True


def take_from_two(count, pile_index_1, pile_index_2, board):
    """
    Pokusí se odebrat daný počet zápalek z daných hromádek.
    :param count: počet odebíraných zápalek
    :param pile_index_1: index první hromádky
    :param pile_index_2: index druhé hromádky
    :param board: aktuální stav hry
    :return: True pokud se podařilo
             False pokud se nepodařilo
    """
    if not baze_znalosti.can_take_from_two(count, pile_index_1, pile_index_2, board):
        return False
    board[pile_index_1] -= count
    board[pile_index_2] -= count
    return True


def check_input_pc(input):
    """
    Kontrola vstupu pro počet hromádek.
    :param input: vstup
    :return: True pokud prošla
             False pokud neprošla
    """
    if not input.isdigit():
        return False
    input_int = int(input)
    if input_int < 1:
        return False
    return True


def check_input_mc(input):
    """
    Kontrola vstupu pro počet zápalek.
    :param input: vstup
    :return: True pokud prošla
             False pokud neprošla
    """
    if not input.isdigit():
        return False
    input_int = int(input)
    if input_int not in range(1, max(baze_dat.board)+1):
        return False
    return True


def check_input_pn(input):
    """
    Kontrola vstupu pro čísla hromádek.
    :param input: vstup
    :return: True pokud prošla
             False pokud neprošla
    """
    pile_numbers = input.split(" ")
    for pn in pile_numbers:
        if not pn.isdigit():
            return False
        if int(pn) not in range(1, len(baze_dat.board)+1):
            return False
    return True
