"""
Description: Tento modul komunikuje s uživatelem.
Author: Václav Ovsík
E-mail: ovsikv@students.zcu.cz
Version: 1.0
Date: 18.12.2023
"""
import inferencni_modul


def start_info():
    """
    Vypíše úvodní informace.
    """
    print("Vítejte ve hře NIM s dynamickou nápovědou.")
    print("Hráči střídavě odebírají libovolný počet zápalek z jedné hromádky, \n"
          "případně stejný počet zápalek ze dvou hromádek.\n"
          "Vítězí hráč který odebere poslední zápalku.\n"
          "Ovládání:\n"
          "Počet zápalek je zadáván jako jedno číslo potvrzeno klávesou ENTER.\n"
          "Čísla hromádek jsou zadávána jako jedno číslo, popřípadě jako dvě čísla oddělené mezerou.\n"
          "Níže se vás zeptáme na počet hromádek na hrací ploše.\n")
    divider()


def end_info():
    """
    Zeptá se hráče zda chce hrát znovu.
    :return: odpověď
    """
    divider()
    return input("Chcete hrát znovu? (a/n):")


def user_input_pc():
    """
    Získá vstup od uživatele pro počet hromádek.
    :return: počet hromádek
    """
    # pile_count = int(input("Zadejte počet hromádek: "))
    pile_count = input("Zadejte počet hromádek: ")
    while not inferencni_modul.check_input_pc(pile_count):
        input_error()
        pile_count = input("Zadejte počet hromádek: ")
    return int(pile_count)


def user_input_mc():
    """
    Získá vstup od uživatele pro počet zápalek k odebrání.
    :return: počet zápalek k odebrání
    """
    match_count = input("Zadejte počet zápalek k odebrání: ")
    while not inferencni_modul.check_input_mc(match_count):
        input_error()
        match_count = input("Zadejte počet zápalek k odebrání: ")
    return int(match_count)


def user_input_pn():
    """
    Získá vstup od uživatele pro čísla hromádek ze kterých se bude odebírat.
    :return: čísla hromádek
    """
    pile_number_str = input("Zadejte čísla hromádek/hromádky: ")
    while not inferencni_modul.check_input_pn(pile_number_str):
        input_error()
        pile_number_str = input("Zadejte čísla hromádek/hromádky: ")
    pile_numbers = [int(n)-1 for n in pile_number_str.split(" ")]
    return pile_numbers


def divider():
    """
    Vypíše oddělovač.
    """
    print("-----------------------------------------------------------------------------------------")


def input_error():
    """
    Vypíše chybové hlášení pro chybně zadaný vstup
    """
    print("Chybně zadaný vstup.")


def print_board(board):
    for i, row in enumerate(board):
        print(str(i+1) + ". " + ((max(board) - row) // 2) * " ", end="")
        print(row * "|")
    divider()


def print_opponent_move(pile_indices, count_removed):
    print("Tah protivníka: ")
    if len(pile_indices) == 1:
        print("Protivník odebral {}x zápalku z hromádky číslo {}.".format(count_removed, pile_indices[0]+1))
    else:
        print("Protivník odebral {}x zápalku z hromádek číslo {} a {}.".format(
            count_removed, pile_indices[0] + 1, pile_indices[1] + 1))


def print_winner(winner):
    if winner:
        print("Vyhráli jste.")
    else:
        print("Prohráli jste.")
