"""
Author: Lukas Runt
Email: lrunt@students.zcu.cz
Date: November 22, 2023
Version: 1.0.0
Description: Explanation module, explains how the result was reached
"""

import KomunikacniModul


def explain_solution_procedure(stack):
    """
    Method explains the solution procedure in nature language
    :param stack: stack with procedure
    :return: The procedure
    """
    if len(stack) == 0:
        return "Bylo zadáno vyřešené sudoku."
    else:
        process = "Postup k vyřešení sudoku je následující:\n"
        for step in range(len(stack)):
            if len(stack[step][2]) == 1:
                process += KomunikacniModul.possible_only_one_number(step + 1, stack[step][0], stack[step][2][0])
            else:
                if stack[step][1] != stack[step][2][0]:
                    process += KomunikacniModul.possible_more_numbers_some_tested(step + 1, stack[step][0],
                                                                                  get_array_string(stack[step][2]),
                                                                                  get_numbers_before(stack[step][2],
                                                                                                     stack[step][1]),
                                                                                  stack[step][1])
                else:
                    process += KomunikacniModul.possible_more_numbers(step + 1, stack[step][0],
                                                                      get_array_string(stack[step][2]), stack[step][1])
        return process


def get_array_string(array):
    """
    Returns string representation of array
    :param array: array what we are want to print
    :return: string representation of array
    """
    string = ""
    for element in array:
        string += str(element) + ", "
    return string[:-2]


def get_numbers_before(array, number):
    """
    Returns numbers before given number in string
    :param array: array what we want to print
    :param number: number we want to print to
    :return: string of numbers before given number
    """
    string = ""
    for element in array:
        if element == number:
            break
        else:
            string += str(element) + ", "
    return string[:-2]


def get_array_after(array, number):
    """
    Method returns array after given numbers
    :param array: array where we are looking for subarray
    :param number: number where we will split hte array
    :return: subarray after given number
    """
    for i in range(len(array)):
        if array[i][0] == number:
            return array[i + 1:]
    return array


def get_step(array, number):
    """
    Method for getting step
    :param array: array where I am looking for right step
    :param number: number of field what we are looking for
    :return: number of step
    """
    step = 1
    for i in range(len(array)):
        if array[i][0] == number:
            return step
        step += 1
    return step


def explain_tile_value_choice(coordinates, sudoku):
    """
    Method explains why the value was chosen
    """
    index = coordinates_to_index(coordinates)
    if sudoku.graph.nodes[index].defined:
        return KomunikacniModul.value_entered_by_user(coordinates, sudoku.graph.nodes[index].value)
    else:
        explanation = ""
        fields_resolved_after_field = get_array_after(sudoku.stack, index)
        for field in fields_resolved_after_field:
            sudoku.graph.nodes[field[0]].value = KomunikacniModul.blank_field
        index_value = sudoku.graph.nodes[index].value
        sudoku.graph.nodes[index].value = KomunikacniModul.actual_position
        explanation += KomunikacniModul.sudoku_before_solution(coordinates, get_step(sudoku.stack, index))
        explanation += sudoku.toStr()
        explanation += explain_values(sudoku, index, index_value)
        for field in fields_resolved_after_field:
            sudoku.graph.nodes[field[0]].value = field[1]
        sudoku.graph.nodes[index].value = index_value
        return explanation


def coordinates_to_index(coordinates):
    """
    coordinates to index in array conversion
    :param coordinates: coordinates of field for example: G6
    :return: index in array
    """
    row = coordinates[1]
    column = coordinates[0]
    column_index = ord(column.upper()) - ord('A')
    row_index = int(row)
    return row_index * 9 + column_index


def explain_values(sudoku, index, index_value):
    """
    Methods explains which colors can be used in field with index
    :param index_value: what value wad assigned to the field
    :param sudoku: sudoku where we are looking for result
    :param index: of which field we are looking for result
    :return: explanation what colors we can use
    """
    explanation = ""
    all_possible_colors = [1, 2, 3, 4, 5, 6, 7, 8, 9]
    neighbours = sudoku.graph.nodes[index].neighbours
    explanation += colors_in_row_or_column(neighbours, sudoku, all_possible_colors, "V řádku", [0, 8])
    explanation += colors_in_row_or_column(neighbours, sudoku, all_possible_colors, "Ve sloupci", [8, 15])
    explanation += colors_in_grid(index, sudoku, all_possible_colors)
    if len(all_possible_colors) == 1:
        explanation += KomunikacniModul.one_possible_color(all_possible_colors[0])
        explanation += ""
    else:
        explanation += KomunikacniModul.more_possible_colors(get_array_string(sorted(all_possible_colors)))
    if index_value == all_possible_colors[0]:
        explanation += KomunikacniModul.what_algorithm_assigned_simple(index_value)
    else:
        explanation += KomunikacniModul.what_algorithm_assigned_complicated(index_value,
                                                                            get_numbers_before(all_possible_colors,
                                                                                               index_value))
    return explanation


def colors_in_row_or_column(neighbours, sudoku, all_possible_colors, where, index_range):
    """
    Explains what colors are connected with our field
    :param neighbours: neighbours of field
    :param sudoku: sudoku where we are looking for result
    :param all_possible_colors: are all possible colors in field
    :param where: in which direction we are looking for result
    :param index_range: in which indexes we are looking
    :return: explanation, which colors are connected
    """
    colors = []
    for i in range(index_range[0], index_range[1]):
        neighbour_value = sudoku.graph.nodes[neighbours[i]].value
        if neighbour_value != KomunikacniModul.blank_field and neighbour_value not in colors:
            colors.append(neighbour_value)
    add_colors(all_possible_colors, colors)
    if len(colors) == 0:
        return KomunikacniModul.get_explanation_for_no_color(where)
    elif len(colors) == 1:
        return KomunikacniModul.get_explanation_for_one_color(where, colors[0])
    else:
        return KomunikacniModul.get_explanation_for_more_colors(where, get_array_string(sorted(colors)))


def colors_in_grid(index, sudoku, all_possible_colors):
    """
    Explains which colors are connected in the grid with our field
    :param index: Index of our field
    :param sudoku: sudoku where we are looking for result
    :param all_possible_colors: all possible colors in the field
    :return: explanation which colors are connected with our field
    """
    colors_grid = []
    for i in sudoku.graph.nodes[index].grid:
        neighbour_value = sudoku.graph.nodes[i].value
        if neighbour_value != KomunikacniModul.blank_field and neighbour_value != KomunikacniModul.actual_position and neighbour_value not in colors_grid:
            colors_grid.append(neighbour_value)
    add_colors(all_possible_colors, colors_grid)
    if len(colors_grid) == 0:
        return KomunikacniModul.get_explanation_for_no_color("Ve čtverci 3x3")
    elif len(colors_grid) == 1:
        return KomunikacniModul.get_explanation_for_one_color("Ve čtverci 3x3", colors_grid[0])
    else:
        return KomunikacniModul.get_explanation_for_more_colors("Ve čtverci 3x3", get_array_string(sorted(colors_grid)))


def add_colors(array, colors):
    """
    Counts which colors can be assigned to the field
    :param array: array of possible colors
    :param colors: colors that are not possible to be assigned
    """
    for color in colors:
        if color in array:
            array.remove(color)
