# ------------------------------------------------------------------------------------------------
# Autor: Dominik Vladar
# E-mail: vladar21@students.zcu.cz
# Datum: 17. 12. 2023
# Verze: 1.0
# Semestralni prace z predmetu KIV/UZI
# Zapadoceska univerzita v Plzni, Fakulta aplikovanych ved
# ------------------------------------------------------------------------------------------------
# Tento modul zjisti, pro jakou kombinaci promennych je vyraz splnitelny.
# Pouziva k tomu 2 dulezite postupy: zjednodusovani vyrazu a nahrazovani promennych konstantami.
# ------------------------------------------------------------------------------------------------
import baze_dat
from baze_dat import *
import baze_znalosti
import komunikacni_modul


# Spusti program a obslouzi vstup od uzivatele.
def spust_program():
    while (vyraz := komunikacni_modul.zeptej_se_na_vyraz()) != "konec":
        baze_dat.promenne = {}
        # ziskame jednotlive vyrazy oddelene mezerami
        rozdelene = vyraz.split()
        if len(rozdelene) == 0:
            komunikacni_modul.spatny_vstup()
            continue
        elif rozdelene[0] == "vysledek":
            # prevedeme vyraz na lepe manipulovatelnou strukturu vnorenych vyrazu
            zanorene = ziskej_zanorene_vyrazy(vyraz.replace("vysledek ", "", 1))
            # zjednodusime vyraz tak, jak to jen jde
            zjednoduseni, zjednoduseno = ziskej_vysledek(zanorene)
            promenne_po_zjednoduseni = ziskej_promenne_z_formule(zjednoduseni).values()
            try:
                vysledek = vyhodnot_formuli(zjednoduseni)
                komunikacni_modul.vysvetlit_vysledek(zjednoduseni, vysledek, zjednoduseno, promenne_po_zjednoduseni)
            except Exception as e:
                # vyjimka je vyhozena, pokud uzivatel zada nevalidni vstup (napr "x and or xor not")
                komunikacni_modul.nespravny_vyraz()
        elif rozdelene[0] == "nacist":
            if len(rozdelene) != 2:
                # spatny pocet argumentu
                komunikacni_modul.spatny_vstup()
                continue
            try:
                # ziskame seznam vsech radku, kde 1 radek reprezentuje 1 vyraz
                radky = komunikacni_modul.ziskej_radky_souboru(rozdelene[1])
                for radek in radky:
                    komunikacni_modul.vypis_vyraz(radek)
                    # zjednodusime vyraz
                    zjednoduseni, zjednoduseno = ziskej_vysledek(ziskej_zanorene_vyrazy(radek))
                    promenne_po_zjednoduseni = ziskej_promenne_z_formule(zjednoduseni).values()
                    try:
                        vysledek = vyhodnot_formuli(zjednoduseni)
                        komunikacni_modul.vysvetlit_vysledek(zjednoduseni, vysledek, zjednoduseno,
                                                             promenne_po_zjednoduseni)
                    except Exception as e:
                        komunikacni_modul.nespravny_vyraz()
                    komunikacni_modul.pokracuj_po_enteru()
            except Exception as e:
                komunikacni_modul.souborova_chyba()
        elif rozdelene[0] == "napoveda":
            komunikacni_modul.zobraz_napovedu()
        else:
            komunikacni_modul.spatny_vstup()


# Ziska z vyrazu informace o zanorenych vyrazech v zavorkach
# a vrati strukturu, ktera umozni spravne pracovat s jejich vyssi prioritou.
def ziskej_zanorene_vyrazy(vyraz):
    # musime zajistit mezery kolem zavorek
    tokeny = vyraz.replace("(", " ( ").replace(")", " ) ").split()
    # druha navratova hodnota nas nezajima
    k_vraceni, _ = ziskej_vyraz(tokeny)
    return k_vraceni


# Ziska jeden vyraz uvnitr zavorek (funguje i pro vyraz neobsahujici zavorky).
# Pokud zjisti, ze vyraz obsahuje podvyraz v zavorce, zavola se rekurzivne a tento podvyraz ulozi
# do seznamu, ktery reprezentuje prave zpracovavany vyraz - vytvari se tak vlastne stromova struktura.
# Vraci 2 prvky - nacteny vyraz znazorneny seznamem a index, na kterem funkce prestala zpracovavat
# - to se hodi pro rekurzivni volani.
def ziskej_vyraz(tokeny):
    vyrazy = []
    i = 0
    while i < len(tokeny):
        if tokeny[i] == "(":
            # rekurzivne ziskame vyraz v zavorkach
            podvyraz, ii = ziskej_vyraz(tokeny[i+1:])
            i += ii
            vyrazy.append(podvyraz)
        elif tokeny[i] == ")":
            # vyraz v zavorkach konci
            i += 1
            break
        else:
            vyrazy.append(ziskej_jeden_prvek_vyrazu(tokeny[i]))
            i += 1
    return vyrazy, i + 1


# Ziska jeden atomicky prvek vyrazu.
def ziskej_jeden_prvek_vyrazu(token):
    # bud se jedna o preddefinovany prvek, nebo je to promenna
    for prvek in PREDDEFINOVANE_PRVKY:
        if prvek.reprezentace == token.lower():
            return prvek
    # pridame promennou, pokud o ni jeste nemame zaznam
    promenna = PrvekVyrazu("promenna", token.lower())
    baze_dat.promenne[token.lower()] = promenna
    return promenna


# Nahradi ve vyrazu hledane hodnoty za nove hodnoty.
# Pokud narazi na podseznam, zavola se rekurzivne.
# Vraci dvojici noveho seznamu s nahrazenymi hodnotami a informaci
# o tom, jestli se podarilo vyraz zjednodusit.
def nahradit(seznam, hledane_hodnoty, nova_hodnota):
    if not isinstance(seznam, list):
        # chceme nahradit sekvenci hodnot v prvku a nikoliv seznamu, to ale neni mozne
        return seznam, False
    novy_seznam = []
    zjednoduseno = False
    i = 0
    while i < len(seznam):
        if seznam[i:i+len(hledane_hodnoty)] == hledane_hodnoty and not zjednoduseno:
            # nasli jsme v seznamu odpovidajici sekvenci, kterou je mozne nahradit
            novy_seznam.extend(nova_hodnota)
            i += len(hledane_hodnoty)
            zjednoduseno = True
        elif isinstance(seznam[i], list) and not zjednoduseno:
            # zkusime hledat v podseznamu
            podseznam, zj = nahradit(seznam[i], hledane_hodnoty, nova_hodnota)
            if not zjednoduseno:
                zjednoduseno = zj
            novy_seznam.append(podseznam)
            i += 1
        else:
            # tento prvek neni mozne nahradit
            novy_seznam.append(seznam[i])
            i += 1
    if len(novy_seznam) == 1:
        # seznam obsahuje jediny prvek, zbavime se zavorek
        novy_seznam = novy_seznam[0]
    return novy_seznam, zjednoduseno


# Zjednodusi predanou formuli tak, jak jen je to mozne
# - pokud uz neni mozne zredukovat promenne, vrati zjednoduseny vyraz,
# pokud promenne je mozne zredukovat nebo jsou jiz zredukovany, vrati
# True pokud je vyraz pravdivy, jinak False.
def ziskej_vysledek(zanorene_vyrazy):
    upraveny_vyraz = False

    zjednoduseno = True
    # pokud se podarilo v minule iteraci zjednodusit, budeme se pokouset zjednodusovat dal
    while zjednoduseno:
        zjednoduseno = False
        for pravidlo in baze_znalosti.prepisovaci_pravidla:
            if PROMENNA in pravidlo.puvodni_vyraz or PROMENNA in pravidlo.prepsany_vyraz:
                # v prepisovacim pravidlu jsme nasli promennou, je nutne nahradit skutecnymi nazvy promennych
                for promenna in baze_dat.promenne.values():
                    # rovnou ziskany vyraz zkusime zjednodusit
                    zanorene_vyrazy, zj = nahradit(
                        zanorene_vyrazy,
                        [x if x != PROMENNA else promenna for x in pravidlo.puvodni_vyraz],
                        [x if x != PROMENNA else promenna for x in pravidlo.prepsany_vyraz]
                    )
                    if not zjednoduseno:
                        zjednoduseno = zj
                    else:
                        upraveny_vyraz = True
            else:
                # zjednodusime vyraz pro prepisovaci pravidlo neobsahujici promennou
                zanorene_vyrazy, zj = nahradit(
                    zanorene_vyrazy,
                    pravidlo.puvodni_vyraz,
                    pravidlo.prepsany_vyraz
                )
                if not zjednoduseno:
                    zjednoduseno = zj
                else:
                    upraveny_vyraz = True

    return zanorene_vyrazy, upraveny_vyraz


# Pomoci rekurzivniho volani ziska z vnorenych vyrazu
# vsechny promenne.
def ziskej_promenne_z_formule(podformule):
    # promenne pridavame do slovniku
    k_vraceni = {}
    if not isinstance(podformule, list):
        if podformule.typ == "promenna":
            k_vraceni[podformule.reprezentace] = podformule
        return k_vraceni
    # podformule je seznamem, pres ktery iterujeme
    for prvek in podformule:
        if isinstance(prvek, list):
            # prida vsechny promenne naleznute v rekurzivnim volani
            k_vraceni.update(ziskej_promenne_z_formule(prvek))
        elif prvek.typ == "promenna":
            k_vraceni[prvek.reprezentace] = prvek
    return k_vraceni


# Nahradi prvni promennou ve vyrazu postupne za True a za False,
# a v obou techto variantach zjednodusi vyraz, a rekurzivne se zavola
# pro dalsi promennou - timto pristupem zjistime, v jakych kombinacich musi
# byt hodnoty promennych, aby byl vyraz pravdivy.
# Vraci [nazev_promenne, podminky_pro_pravdive_ohodnoceni_pokud_je_false,
#       podminky_pro_pravdive_ohodnoceni_pokud_je_true]
def vyhodnot_formuli(zjednoduseno):
    if not isinstance(zjednoduseno, list):
        # zjednoduseno je jiz vyhodnoceno jako true nebo false, nebo konstanta
        # pokud ne, tak se jedna o chybny zapis
        if zjednoduseno.typ == "binarni operator" or zjednoduseno.typ == "unarni operator":
            raise Exception("spatny vstup")
        zjednoduseno = [zjednoduseno]
    # projdeme zjednodusenou formuli a najdeme vsechny promenne
    # (nektere jiz mohly byt zredukovany)
    promenne_po_zjednoduseni = ziskej_promenne_z_formule(zjednoduseno)
    if len(promenne_po_zjednoduseni) == 0:
        if len(zjednoduseno) != 1 or (zjednoduseno[0] != TRUE and zjednoduseno[0] != FALSE):
            raise Exception("spatny vstup")
        return zjednoduseno[0] == TRUE
    # nasledne kazde promenne priradime hodnotu TRUE a FALSE (vsechny kombinace)
    # vlastne muzeme postupovat rekurzivne, za jednu promennou dosadime TRUE, FALSE a zavolame vyhodnot_formuli
    zjednodusovat = True
    nahrazeno_true = nahrazeno_false = zjednoduseno
    # dosadime do obou formuli true/false a zjednodusujeme je, dokud je to mozne
    while zjednodusovat:
        nahrazeno_true, zjednodusovat = nahradit(nahrazeno_true, [list(promenne_po_zjednoduseni.values())[0]], [TRUE])
    zjednodusovat = True
    while zjednodusovat:
        nahrazeno_false, zjednodusovat = nahradit(nahrazeno_false,
                                                  [list(promenne_po_zjednoduseni.values())[0]], [FALSE])

    nahrazeno_true, _ = ziskej_vysledek(nahrazeno_true)
    nahrazeno_false, _ = ziskej_vysledek(nahrazeno_false)

    false_strana = vyhodnot_formuli(nahrazeno_false)
    true_strana = vyhodnot_formuli(nahrazeno_true)
    if isinstance(false_strana, list) or isinstance(true_strana, list) or false_strana or true_strana:
        # spravny vstup od uzivatele
        return [list(promenne_po_zjednoduseni)[0], false_strana, true_strana]
    # nespravny vstup, uzivatel zadal spatny vyraz, napr. (xor or xor (or))
    return False
